unit uOptimalne;

interface

uses
  MSXML2_TLB, contnrs, SysUtils, Classes;

type
  TKriterium = (kNejkratsi, kNejrychlejsi, kNejlevnejsi);
  TMesta = class;
  TMesto = class
    Nazev: string;
    Minimum: Double;
    Owner: TMesta;
    Hledane: Boolean;
    constructor Create(const ANazev: string; AOwner: TMesta);
    function ZjistiHodnotu(Kriterium: TKriterium; Cesta: IXMLDOMNode): Double;
    function HledejCestu(Hodnota: Double; Kriterium: TKriterium): Double;
  end;

  TMesta = class(TObjectList)
    Mapa: IXMLDomDocument;
    Minimum: Double;
    Cesta: TList;
    constructor Create(AMapa: IXMLDomDocument);
    function MestoPodleJmena(const Jmeno: string): TMesto;
    procedure Nacti;
  end;

implementation

constructor TMesto.Create;
begin
  Minimum := MaxInt;
  Nazev := ANazev;
  Owner := AOwner;
end;

function TMesto.ZjistiHodnotu;
var
  Rychlost, Vzdalenost: Integer;
  Spotreba: Double;
  Separator: Char;
begin
  case Kriterium of
    kNejkratsi: Result := Cesta.selectSingleNode('vzdalenost').nodeTypedValue;
    kNejrychlejsi: Result := Cesta.selectSingleNode('vzdalenost').nodeTypedValue /
       Cesta.selectSingleNode('prumernaRychlost').nodeTypedValue;
    kNejlevnejsi: begin
      Rychlost := Cesta.selectSingleNode('prumernaRychlost').nodeTypedValue;
      Separator := DecimalSeparator;
      DecimalSeparator := '.';
      Spotreba := StrToFloat(Cesta.ownerDocument.selectSingleNode(Format('//spotreba[rychlost="%d"]/litru', [Rychlost])).nodeTypedValue);
      DecimalSeparator := Separator; 
      Vzdalenost := Cesta.selectSingleNode('vzdalenost').nodeTypedValue;
      Result := (Spotreba / 100) * Vzdalenost;
    end;
  end;
end;

function TMesto.HledejCestu;
var
  x: Double;
  i: Integer;
  s: string;
  cesty: IXMLDOMNodeList;
begin
  if (Hodnota < Minimum) and (Hodnota < Owner.Minimum) then
  begin
    Minimum := Hodnota;
    if Hledane then
    begin
      Owner.Minimum := Hodnota;
      Result := Hodnota;
      Owner.Cesta.Clear;
      Owner.Cesta.Add(Self);
    end else
    begin
      Result := MaxInt;
      cesty := Owner.Mapa.selectNodes(Format('//cesta[z="%s"]', [Nazev]));
      for i := 0 to cesty.length - 1 do
      begin
        s := cesty.item[i].selectSingleNode('do').nodeTypedValue;
        x := Owner.MestoPodleJmena(s).HledejCestu(Hodnota + ZjistiHodnotu(Kriterium, cesty.item[i]), Kriterium);
        if x = Owner.Minimum then
        begin
          Result := x;
          Owner.Cesta.Add(Self);
        end;
      end;
      cesty := Owner.Mapa.selectNodes(Format('//cesta[do="%s"]', [Nazev]));
      for i := 0 to cesty.length - 1 do
      begin
        s := cesty.item[i].selectSingleNode('z').nodeTypedValue;
        x := Owner.MestoPodleJmena(s).HledejCestu(Hodnota + ZjistiHodnotu(Kriterium, cesty.item[i]), Kriterium);
        if x = Owner.Minimum then
        begin
          Result := x;
          Owner.Cesta.Add(Self);
        end;
      end;
    end;
  end;
end;

function TMesta.MestoPodleJmena;
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
  begin
    if Jmeno = TMesto(Items[i]).Nazev then
    begin
      Result := TMesto(Items[i]);
      Exit;
    end;
  end;
end;

constructor TMesta.Create;
begin
  inherited;
  Mapa := AMapa;
  Minimum := MaxInt;
  Cesta := TList.Create;
end;

procedure TMesta.Nacti;
var
  i: Integer;
  mesta: IXMLDOMNodeList;
begin
  mesta := Mapa.selectNodes('//mesto');
  for i := 0 to mesta.length - 1 do
    Add(TMesto.Create(mesta.item[i].selectSingleNode('nazev').nodeTypedValue, Self));
end;

end.
